#define TK_END_OF_FILE                     1
#define TK_ILLEGAL                         2
#define TK_SPACE                           3
#define TK_UNCLOSED_STRING                 4
#define TK_COMMENT                         5
#define TK_FUNCTION                        6
#define TK_COLUMN                          7
#define TK_AGG_FUNCTION                    8
#define TK_CONST_FUNC                      9
#define TK_SEMI                           10
#define TK_EXPLAIN                        11
#define TK_BEGIN                          12
#define TK_TRANSACTION                    13
#define TK_DEFERRED                       14
#define TK_IMMEDIATE                      15
#define TK_EXCLUSIVE                      16
#define TK_COMMIT                         17
#define TK_END                            18
#define TK_ROLLBACK                       19
#define TK_CREATE                         20
#define TK_TABLE                          21
#define TK_TEMP                           22
#define TK_LP                             23
#define TK_RP                             24
#define TK_AS                             25
#define TK_COMMA                          26
#define TK_ID                             27
#define TK_ABORT                          28
#define TK_AFTER                          29
#define TK_ASC                            30
#define TK_ATTACH                         31
#define TK_BEFORE                         32
#define TK_CASCADE                        33
#define TK_CONFLICT                       34
#define TK_DATABASE                       35
#define TK_DESC                           36
#define TK_DETACH                         37
#define TK_EACH                           38
#define TK_FAIL                           39
#define TK_FOR                            40
#define TK_IGNORE                         41
#define TK_INITIALLY                      42
#define TK_INSTEAD                        43
#define TK_LIKE_KW                        44
#define TK_MATCH                          45
#define TK_KEY                            46
#define TK_OF                             47
#define TK_OFFSET                         48
#define TK_PRAGMA                         49
#define TK_RAISE                          50
#define TK_REPLACE                        51
#define TK_RESTRICT                       52
#define TK_ROW                            53
#define TK_STATEMENT                      54
#define TK_TRIGGER                        55
#define TK_VACUUM                         56
#define TK_VIEW                           57
#define TK_REINDEX                        58
#define TK_RENAME                         59
#define TK_CTIME_KW                       60
#define TK_ALTER                          61
#define TK_OR                             62
#define TK_AND                            63
#define TK_NOT                            64
#define TK_IS                             65
#define TK_BETWEEN                        66
#define TK_IN                             67
#define TK_ISNULL                         68
#define TK_NOTNULL                        69
#define TK_NE                             70
#define TK_EQ                             71
#define TK_GT                             72
#define TK_LE                             73
#define TK_LT                             74
#define TK_GE                             75
#define TK_ESCAPE                         76
#define TK_BITAND                         77
#define TK_BITOR                          78
#define TK_LSHIFT                         79
#define TK_RSHIFT                         80
#define TK_PLUS                           81
#define TK_MINUS                          82
#define TK_STAR                           83
#define TK_SLASH                          84
#define TK_REM                            85
#define TK_CONCAT                         86
#define TK_UMINUS                         87
#define TK_UPLUS                          88
#define TK_BITNOT                         89
#define TK_STRING                         90
#define TK_JOIN_KW                        91
#define TK_CONSTRAINT                     92
#define TK_DEFAULT                        93
#define TK_NULL                           94
#define TK_PRIMARY                        95
#define TK_UNIQUE                         96
#define TK_CHECK                          97
#define TK_REFERENCES                     98
#define TK_COLLATE                        99
#define TK_AUTOINCR                       100
#define TK_ON                             101
#define TK_DELETE                         102
#define TK_UPDATE                         103
#define TK_INSERT                         104
#define TK_SET                            105
#define TK_DEFERRABLE                     106
#define TK_FOREIGN                        107
#define TK_DROP                           108
#define TK_UNION                          109
#define TK_ALL                            110
#define TK_INTERSECT                      111
#define TK_EXCEPT                         112
#define TK_SELECT                         113
#define TK_DISTINCT                       114
#define TK_DOT                            115
#define TK_FROM                           116
#define TK_JOIN                           117
#define TK_USING                          118
#define TK_ORDER                          119
#define TK_BY                             120
#define TK_GROUP                          121
#define TK_HAVING                         122
#define TK_LIMIT                          123
#define TK_WHERE                          124
#define TK_INTO                           125
#define TK_VALUES                         126
#define TK_INTEGER                        127
#define TK_FLOAT                          128
#define TK_BLOB                           129
#define TK_REGISTER                       130
#define TK_VARIABLE                       131
#define TK_EXISTS                         132
#define TK_CASE                           133
#define TK_WHEN                           134
#define TK_THEN                           135
#define TK_ELSE                           136
#define TK_INDEX                          137
#define TK_TO                             138
#define TK_ADD                            139
#define TK_COLUMNKW                       140
